package com.aliyun.chatbot.channel.wechat;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.wechat.aes.WXBizMsgCrypt;
import com.aliyun.chatbot.channel.wechat.work.WechatWorkChatClient;
import com.aliyun.chatbot.channel.wechat.work.WechatWorkInputMsg;
import com.aliyun.chatbot.channel.wechat.work.WechatWorkTokenClient;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.google.common.collect.Maps;
import lombok.val;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpMethod;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author kelin
 * @date 2024/8/7
 */

@RestController
public class WechatCallbackController {
    private static final Logger logger = LoggerFactory.getLogger(WechatCallbackController.class);

    @Autowired
    private WechatWorkChatClient wechatWorkChatClient;

    @Value("${WX_CORP_ID}")
    private String CORP_ID;

    @Value("${WX_TOKEN}")
    private String TOKEN;

    @Value("${WX_ENCODING_AES_KEY}")
    private String ENCODING_AES_KEY;

    @Value("${WX_SECRET}")
    private String SECRET;

    @RequestMapping("/api/wechat/work/chat")
    public String message(HttpServletRequest request, @RequestBody(required = false) String postData) {
        try {
            Map<String, String> parameter = getParameter(request);
            logger.info("[微信消息处理],消息原文:{}, parameter:{}", request.getRequestURL(), JSON.toJSONString(parameter));
            if (HttpMethod.GET.matches(request.getMethod())) {
                return verifyURL(parameter);
            } else {
                return callback(request, postData);
            }
        } catch (Exception e) {
            logger.error("[微信消息处理异常],消息原文:{}", request.getRequestURI(), e);
        }
        return DEFAULT_RETURN_STR;
    }

    private String verifyURL(Map<String, String> parameter) throws Exception {

        String signature = parameter.get("msg_signature");
        String timestamps = parameter.get("timestamp");
        String nonce = parameter.get("nonce");
        String echoStr = parameter.get("echostr");

        WXBizMsgCrypt wxBizJsonMsgCrypt = new WXBizMsgCrypt(TOKEN, ENCODING_AES_KEY, CORP_ID);
        return wxBizJsonMsgCrypt.VerifyURL(signature, timestamps, nonce, echoStr);
    }

    public String callback(HttpServletRequest request, String postData) throws Exception {

        val msg_signature = request.getParameter("msg_signature");
        val nonce = request.getParameter("nonce");
        val timestamp = request.getParameter("timestamp");

        WechatWorkTokenClient wxTokenClient = new WechatWorkTokenClient(CORP_ID, SECRET);

        WXBizMsgCrypt wxBizMsgCrypt = new WXBizMsgCrypt(TOKEN, ENCODING_AES_KEY, CORP_ID);
        String sMsg = wxBizMsgCrypt.DecryptMsg(msg_signature, timestamp, nonce, postData);

        XmlMapper xmlMapper = new XmlMapper();
        WechatWorkInputMsg msg = xmlMapper.readValue(sMsg, WechatWorkInputMsg.class);

        if (msg.getContent() != null && msg.getMsgType() != null && "text".equals(msg.getMsgType())) {
            wechatWorkChatClient.chat(msg, wxTokenClient);
            return "success";
        } else {
            return "success";
        }

    }

    private Map<String, String> getParameter(HttpServletRequest request) {
        Map<String, String> parameter = Maps.newHashMap();
        Map<String, String[]> parameterMap = request.getParameterMap();
        if (CollectionUtils.isEmpty(parameterMap)) {
            return parameter;
        }
        parameterMap.forEach((key, value) -> parameter.put(key, value.length > 0 ? value[0] : null));

        return parameter;
    }

    private static final String DEFAULT_RETURN_STR = "兜底回复";

}