package com.aliyun.chatbot.channel.wechat.work;

import javax.annotation.PostConstruct;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.base.ChannelMsgSender;
import com.aliyun.chatbot.channel.chatbot.ChatbotClient;
import com.aliyun.chatbot.channel.chatbot.ChatbotSSERequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * @author kelin
 * @date 2024/8/9
 */
@Component
@Slf4j
public class WechatWorkChatClient {
    private ChatbotClient chatbotClient;

    @Value("${CHATBOT_INSTANCE_ID}")
    private String CHATBOT_INSTANCE_ID;
    @Value("${CHATBOT_AGENT_KEY}")
    private String CHATBOT_AGENT_KEY;
    @Value("${ALIYUN_ACCESS_KEY}")
    private String ALIYUN_ACCESS_KEY;
    @Value("${ALIYUN_SECRET_KEY}")
    private String ALIYUN_SECRET_KEY;

    @PostConstruct
    public void init() {
        chatbotClient = new ChatbotClient(CHATBOT_AGENT_KEY, ALIYUN_ACCESS_KEY, ALIYUN_SECRET_KEY);
    }

    public void chat(WechatWorkInputMsg msg, WechatWorkTokenClient tokenClient) {
        log.info("receive message :{}", JSON.toJSONString(msg));
        try {
            if (msg != null && "text".equals(msg.getMsgType())) {
                //机器人接收消息内容
                WechatWorkChatContext context = new WechatWorkChatContext();
                context.setMsg(msg);
                context.setWxTokenClient(tokenClient);

                ChannelMsgSender sender = new WechatWorkClientMsgSender(context);
                sender.preSengMsg();
                String sessionId = msg.getFromUserName();

                // 调用大模型
                ChatbotSSERequest reqData = ChatbotSSERequest.build(CHATBOT_INSTANCE_ID, msg.getContent(),
                    sessionId,
                    msg.getMsgId().toString());
                chatbotClient.invokeLlm(reqData, sender);
            }
        } catch (Exception e) {
            log.error("receive message by error:" + e.getMessage(), e);
        }
    }
}
