package com.aliyun.chatbot.channel.wechat.work;

import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.utils.OkHttpUtils;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import lombok.extern.slf4j.Slf4j;

/**
 * @author kelin
 * @date 2024/8/8
 */
@Slf4j
public class WechatWorkTokenClient {
    private final String corpId;
    private final String secret;

    private static final LoadingCache<String, String> CACHE = CacheBuilder.newBuilder()
        .maximumSize(10)
        .expireAfterWrite(90, TimeUnit.MINUTES)
        .build(new CacheLoader<String, String>() {
            @Override
            public String load(String key) throws Exception {
                return doGetToken(key);
            }
        });

    public WechatWorkTokenClient(String corpId, String secret) {
        this.corpId = corpId;
        this.secret = secret;
    }

    public String getToken() throws Exception {
        return CACHE.get(getKey(corpId, secret));
    }

    private static String doGetToken(String key) {
        String tokeUrl = "https://qyapi.weixin.qq.com/cgi-bin/gettoken";

        String resultToken = OkHttpUtils.get(tokeUrl + "?" + key, null);
        String accessToken = String.valueOf(JSON.parseObject(resultToken).getString("access_token"));
        log.info(JSON.toJSONString(resultToken));
        return accessToken;
    }

    private static String getKey(String corpId, String secret) {
        return "corpid=" + corpId + "&corpsecret=" + secret;
    }

}
